#include "font.c"


#define RGB5( r, g, b )	( ( ( b ) << 10 ) + ( ( g ) << 5 ) + ( ( r ) << 0 ) + 0x8000 )
#define R5( c )			( ( ( c ) >> 0 ) & 31 )
#define G5( c )			( ( ( c ) >> 5 ) & 31 )
#define B5( c )			( ( ( c ) >> 10 ) & 31 )


#define AY8(a,y) ( (a<<24) | (y * 0x10101) )
#define RGB888(r,g,b) ( (b<<16)  | (g<<8) | (r) | 0xff000000 )
#define RGB444(r,g,b) ( ((b<<4)&0xf00)  | (g&0xf0) | ((r>>4)&0xf) | 0xf000 )
#define RGB565(r,g,b) ( ((b<<8) & 0xf800) | ((g<<3) & 0x7e0) | ((r>>3) & 0x1F) )

#define WHITE RGB888(255,255,255)
#define GREY  RGB888(127,127,127)
#define RED   RGB888(255,0,0)
#define GREEN RGB888(0,255,0)
#define BLUE  RGB888(0,0,255)
#define BLACK RGB888(0,0,0)
#define YELLOW RGB888(255,255,0)
#define ORANGE RGB888(255,127,0)
#define VIOLET RGB888(255,0,255)
#define TEAL RGB888(0,255,255)

//constants
#define		PIXELSIZE	1				//in short
#define		LINESIZE	512				//in short
#define		FRAMESIZE	0x44000			//in byte

#define SCREEN_WIDTH  480
#define SCREEN_HEIGHT 272

//480*272 = 60*38
#define CMAX_X 60
#define CMAX_Y 38
#define CMAX2_X 30
#define CMAX2_Y 19
#define CMAX4_X 15
#define CMAX4_Y 9

char *pg_vramtop=(char *)0x04000000;
long pg_screenmode;
long pg_showframe;
long pg_drawframe;

//-------------------------------------------------------------------------------------------------
char *GetVramAddr(unsigned long x,unsigned long y)
{
	return pg_vramtop+(pg_drawframe?FRAMESIZE:0)+x*PIXELSIZE*2+y*LINESIZE*2+0x40000000;
}
//-------------------------------------------------------------------------------------------------
void pgWaitV()
{
	sceDisplayWaitVblankStart();
}
//-------------------------------------------------------------------------------------------------
void InitScreen()
{
	sceDisplaySetMode(0,480,272);
	sceDisplaySetFrameBuf(pg_vramtop,512,1,1);
}

//-------------------------------------------------------------------------------------------------
void ClearScreen(u32 color)
{
	unsigned char *vptr0;		//pointer to vram
	unsigned long i;

	vptr0=GetVramAddr(0,0);
	for (i=0; i<0x44000/2; i++) {
		*(unsigned short *)vptr0=color;
		vptr0+=2;
	}
}
/*
//added by Gamblore
//-------------------------------------------------------------------------------------------------
void swapBuffer()
{
	//sceDisplayWaitVblankStart();
	tmp=vramtop;
	vramtop=backbuffer;
	backbuffer=tmp;
}

//added by Gamblore
//-------------------------------------------------------------------------------------------------
void swapActive()
{
	sceDisplayWaitVblankStart();
	sceDisplaySetFrameBuf(vramtop,512,1,1);
}
*/
//-------------------------------------------------------------------------------------------------
void PutPixel(long x, long y, u32 color)
{
	unsigned char *vptr;		//pointer to vram
    
    if(x>479)
      x=480;
    if(x<0)
      x=-1;
    if(y<0)
      y=0;
      
	vptr=GetVramAddr(x,y);
	*(unsigned short *)vptr=color;
}
//-------------------------------------------------------------------------------------------------
inline u16	BlendRGB( u16 rgb1, u16 rgb2, u8 t )
{
	const s32	r1 = R5( rgb1 );
	const s32	g1 = G5( rgb1 );
	const s32	b1 = B5( rgb1 );
	const s32	r2 = R5( rgb2 );
	const s32	g2 = G5( rgb2 );
	const s32	b2 = B5( rgb2 );

	const s32	rd = ( r2 - r1 ) * t;
	const s32	gd = ( g2 - g1 ) * t;
	const s32	bd = ( b2 - b1 ) * t;

	const s32	r = r1 + ( rd >> 8 );
	const s32	g = g1 + ( gd >> 8 );
	const s32	b = b1 + ( bd >> 8 );

	return RGB5( r, g, b );
}
//-------------------------------------------------------------------------------------------------
void PutGfxNoFade(unsigned long x,unsigned long y,unsigned long w,unsigned long h,const unsigned short *d, int centerx, int centery)
{
	unsigned char *vptr0;		//pointer to vram
	unsigned char *vptr;		//pointer to vram
	unsigned long xx,yy;
	const unsigned short *dd;
	
	
	if(centerx!=0){
		x=240-(w/2);
	}


	if(centery!=0){
		y=136-(h/2);
	}
	
	vptr0=GetVramAddr(x,y);
	for (yy=0; yy<h; yy++) {
		vptr=vptr0;
		dd=d;
		for (xx=0; xx<w; xx++) {
			*(unsigned short *)vptr=*dd;
			vptr+=2;
			dd++;
		}
		vptr0+=512*2;
		d+=w;
	}
	
}
//-------------------------------------------------------------------------------------------------
void PutGfx(unsigned long x,unsigned long y,unsigned long w,unsigned long h,const unsigned short *d, int centerx, int centery, unsigned fadefrom, int fadevalue)
{
	unsigned char *vptr0;		//pointer to vram
	unsigned char *vptr;		//pointer to vram
	unsigned long xx,yy;
	const unsigned short *dd;
	
	
	if(centerx!=0){
		x=240-(w/2);
	}


	if(centery!=0){
		y=136-(h/2);
	}
	
	vptr0=GetVramAddr(x,y);
	for (yy=0; yy<h; yy++) {
		vptr=vptr0;
		dd=d;
		for (xx=0; xx<w; xx++) {
			*(unsigned short *)vptr=BlendRGB(fadefrom,*dd,fadevalue);
			vptr+=2;
			dd++;
		}
		vptr0+=512*2;
		d+=w;
	}
	
}
//-------------------------------------------------------------------------------------------------
void PutGfxTrans(unsigned long transcol, unsigned long x,unsigned long y,unsigned long w,unsigned long h,const unsigned short *d, int centerx, int centery, unsigned fadefrom, int fadevalue)
{
	unsigned char *vptr0;		//pointer to vram
	unsigned char *vptr;		//pointer to vram
	unsigned long xx,yy;
	const unsigned short *dd;
	
	
	if(centerx!=0){
		x=240-(w/2);
	}


	if(centery!=0){
		y=136-(h/2);
	}
	
	vptr0=GetVramAddr(x,y);
	for (yy=0; yy<h; yy++) {
		vptr=vptr0;
		dd=d;
		for (xx=0; xx<w; xx++) {
			if(*dd!= transcol){
				*(unsigned short *)vptr=BlendRGB(fadefrom,*dd,fadevalue);
			}
			vptr+=2;
			dd++;
		}
		vptr0+=512*2;
		d+=w;
	}
	
}
//-------------------------------------------------------------------------------------------------
void PutSprite(unsigned long x,unsigned long y,unsigned long w,unsigned long h,unsigned long numspr,const unsigned short *d)
{
 unsigned long xx,yy;
 const unsigned short *dd;
    
    
    
	dd=d;
	dd=dd+w*h*numspr;
	for(xx=0;xx<w;xx++){
	      for(yy=0;yy<h;yy++){
	        PutPixel(x+xx,y+yy,*dd);
            dd++;
          } 
     }
}
//-------------------------------------------------------------------------------------------------
void PutSpriteTrans(unsigned long transcol,unsigned long x,unsigned long y,unsigned long w,unsigned long h,unsigned long numspr,const unsigned short *d)
{
 unsigned long xx,yy;
 const unsigned short *dd;
    
    
    
	dd=d;
	dd=dd+w*h*numspr;
	for(xx=0;xx<w;xx++){
	      for(yy=0;yy<h;yy++){
	        if(*dd!=transcol){
               PutPixel(x+xx,y+yy,*dd);
            }
			dd++;
          } 
     }
}
//-------------------------------------------------------------------------------------------------
void Putbg(unsigned long x,unsigned long y,unsigned long w,unsigned long h,unsigned long mag,const unsigned short *d)
{
	unsigned char *vptr0;		//pointer to vram
	unsigned char *vptr;		//pointer to vram
	unsigned long xx,yy,mx,my;
	const unsigned short *dd;
	
	vptr0=GetVramAddr(x,y);
	for (yy=0; yy<h; yy++) {
		for (my=0; my<mag; my++) {
			vptr=vptr0;
			dd=d;
			for (xx=0; xx<w; xx++) {
				for (mx=0; mx<mag; mx++) {
					*(unsigned short *)vptr=*dd;
					vptr+=2;
				}
				dd++;
			}
			vptr0+=512*2;
		}
		d+=480;
	}
	
}

//-------------------------------------------------------------------------------------------------
void PauseVbl(unsigned int tempo){
	unsigned int i;

	for(i=0;i<=tempo;i++){
		sceDisplayWaitVblankStart();
    }
}

//-------------------------------------------------------------------------------------------------
void PutChar(unsigned long x,unsigned long y,unsigned long color,unsigned long bgcolor,unsigned char ch,char drawfg,char drawbg,char mag)
{
	unsigned char *vptr0;		//pointer to vram
	unsigned char *vptr;		//pointer to vram
	const unsigned char *cfont;		//pointer to font
	unsigned long cx,cy;
	unsigned long b;
	char mx,my;

	cfont=font+ch*8;
	vptr0=GetVramAddr(x,y);
	for (cy=0; cy<8; cy++) {
		for (my=0; my<mag; my++) {
			vptr=vptr0;
			b=0x80;
			for (cx=0; cx<8; cx++) {
				for (mx=0; mx<mag; mx++) {
					if ((*cfont&b)!=0) {
						if (drawfg) *(unsigned short *)vptr=color;
					} else {
						if (drawbg) *(unsigned short *)vptr=bgcolor;
					}
					vptr+=PIXELSIZE*2;
				}
				b=b>>1;
			}
			vptr0+=LINESIZE*2;
		}
		cfont++;
	}
}

//-------------------------------------------------------------------------------------------------
void Print(unsigned long x,unsigned long y,u32 color,const char *str)
{
	while (*str!=0 && x<CMAX_X && y<CMAX_Y) {
		PutChar(x*8,y*8,color,0,*str,1,0,1);
		str++;
		x++;
		if (x>=CMAX_X) {
			x=0;
			y++;
		}
	}
}
void pgScreenFrame(long mode,long frame)
{
	pg_screenmode=mode;
	frame=(frame?1:0);
	pg_showframe=frame;
	if (mode==0) {
		//screen off
		pg_drawframe=frame;
		sceDisplaySetFrameBuf(0,0,0,1);
	} else if (mode==1) {
		//show/draw same
		pg_drawframe=frame;
		sceDisplaySetFrameBuf(pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,1);
	} else if (mode==2) {
		//show/draw different
		pg_drawframe=(frame?0:1);
		sceDisplaySetFrameBuf(pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,1);
	}
}


void pgScreenFlip()
{
	pg_showframe=(pg_showframe?0:1);
	pg_drawframe=(pg_drawframe?0:1);
	sceDisplaySetFrameBuf(pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,0);
}


void pgScreenFlipV()
{
	pgWaitV();
	pgScreenFlip();
}

void pgFlipDrawFrame()
{
	pg_drawframe=(pg_drawframe?0:1);
}

void pgFlipShowFrameV()
{
	pgWaitV();
	pg_showframe=(pg_showframe?0:1);
	sceDisplaySetFrameBuf(pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,0);
}




